#!/bin/bash

prep_dir=voting-excel

if [ ! -d ./my-python3 ] ; then
    echo "" 1>&2
    echo "Failed to find: ./my-python3" 1>&2
    echo "Have you run:" 1>&2
    echo "  ./CREATE-PYTHON-VENV.sh" 1>&2
    echo "" 1>&2
    
    exit 1
fi

if [ ! -d "$prep_dir/metadata-votes" ] ; then
    echo "Creating directory:"
    echo "  $prep_dir/metadata-votes"
    
    mkdir "$prep_dir/metadata-votes"
fi


if [ ! -f "$prep_dir/eurovision_song_contest_1975_2019.xlsx" ] ; then
    echo "Unzipping $prep_dir/archive.zip:"
    
    cd "$prep_dir" && unzip archive.zip && cd ..
fi

#
# Get list of European countries in CSV format
# (used by python script)
#
if [ ! -f voting-excel/dbpedia--european-countries.csv ] ; then
    ./DBPEDIA-LOD-SPARQL-QUERY.sh voting-excel/dbpedia--european-countries.sparql csv
    if [ $? != 0 ] ; then
	echo "Error: Failed to run DBpedia SPARQL query:" 1>&2
	echo "  voting-excel/dbpedia--european-countries.sparql" 1>&2
	exit 1
    fi
else
    echo "Detected:"
    echo "  voting-excel/dbpedia--european-countries.csv"
    echo "=> Skipping DBpedia SPARQL query to retrieve European countries in CSV format"    
fi

exit_status=0

source  ./my-python3/bin/activate

$prep_dir/xlsx-fromcountry-jsonmetadata.py \
    --votingtype "J" \
    $prep_dir/eurovision_song_contest_1975_2019.xlsx  \
    $prep_dir/metadata-votes/metadata-votes-fromcountry-jury.json


if [ $? = 0 ] ; then
    $prep_dir/xlsx-fromcountry-jsonmetadata.py \
	--votingtype "T" \
	$prep_dir/eurovision_song_contest_1975_2019.xlsx  \
	$prep_dir/metadata-votes/metadata-votes-fromcountry-tele.json

    if [ $? = 0 ] ; then

	$prep_dir/xlsx-fromcountry-jsonmetadata.py \
	    --votingtype "JT" \
	    $prep_dir/eurovision_song_contest_1975_2019.xlsx  \
	    $prep_dir/metadata-votes/metadata-votes-fromcountry-comb.json

	if [ $? = 0 ] ; then	
	    $prep_dir/xlsx-tocountry-jsonmetadata.py $prep_dir/eurovision_song_contest_1975_2019.xlsx  $prep_dir/metadata-votes-tocountry.json

	    if [ $? != 0 ] ; then
		echo "!!!!" 1>&2
		echo "! Error enountered generating to-country metadata-json: $prep_dir/metadata-votes-tocountry.json" 1>&2
		echo "!!!!" 1>&2
		exit_status=1
	    fi
	else
	    echo "!!!!" 1>&2
	    echo "! Error enountered generating from-country combined Jury and Televoting metadata-json in $prep_dir/metadata-votes/" 1>&2
	    echo "!!!!" 1>&2
	    exit_status=1
	fi
    else
	echo "!!!!" 1>&2
	echo "! Error enountered generating from-country Televoting metadata-json in $prep_dir/metadata-votes/" 1>&2
	echo "!!!!" 1>&2
	exit_status=1
    fi
else
    echo "!!!!" 1>&2
    echo "! Error enountered generate from-country Jury metadata-json in $prep_dir/metadata-votes/" 1>&2
    echo "!!!!" 1>&2
    exit_status=1
fi


deactivate

if [ $exit_status != 0 ] ; then
    exit $exit_status
fi
