#!/usr/bin/env python

from __future__ import print_function

import os
import re
import sys
import json

import argparse
import openpyxl

import util
import xlsxutil

#jury_from_countries = {}
#tele_from_countries = {}
    
def list_jury_tele_comb_votes(data_hashmap_array):

    data_hashmap_array_filtered = list(filter(xlsxutil.filter_jury_or_tele_votes, data_hashmap_array))
    data_hashmap_array_filtered.sort(key=xlsxutil.sortkey_year_from_country_to_country_voting_type)
    data_hashmap_array_filtered_len = len(data_hashmap_array_filtered)
    
    data_hashmap_array_filtered_comb_pairs = []

    i = 0;
    while (i<data_hashmap_array_filtered_len-1):
        entry = data_hashmap_array_filtered[i];
        next_entry = data_hashmap_array_filtered[i+1];

        if ((entry.get("Year") == next_entry.get("Year")) and (entry.get("From country") == next_entry.get("From country"))  
            and ((entry.get("Jury or Televoting") == "J") and (next_entry.get("Jury or Televoting") == "T"))):

            comb_entry = entry.copy()
            comb_entry["Points"] = entry.get("Points") + next_entry.get("Points")
            comb_entry["Jury or Televoting"] = "JT"

            from_country = entry.get("From country")
            year = entry.get("Year")
            from_country_id = "FromCountry-" + re.sub(r'\s+', '', from_country) + str(year)
                
            comb_entry["FromCountry-Child-Jury"] = from_country_id + "-J"
            comb_entry["FromCountry-Child-Tele"] = from_country_id + "-T"
            
            data_hashmap_array_filtered_comb_pairs.append(comb_entry)
            i = i + 2 
        else:
            i = i + 1
            
    return data_hashmap_array_filtered_comb_pairs


def create_from_country_voting_groups(data_hashmap_array):

    # Example values for header-names
    #   (semi-) final: f, sf
    #   Jury or Televoting: J, T
    #   Year: 1975
    #   To country: Belgium
    #   From country: Belgium
    #   Edition: 1975f, 1975sf

    
    # Sort so array entries are grouped by the country receiving the votes in a given year
    data_hashmap_array.sort(key=xlsxutil.sortkey_year_from_country)


    # Debug output
    #
    # for data_hashmap in data_hashmap_array:
    #    print(data_hashmap.get('From country'), data_hashmap.get('Year'), data_hashmap.get('Points'), "(Points to " + data_hashmap.get('To country') + ")")


    # Build array of country groups
    #   A country grouping includes all the votes that country receive that year
    country_groups = []

    prev_data_hashmap = data_hashmap_array[0]
    country_group = [ ]

    i = 1
    num_rows = len(data_hashmap_array)

    while (i < num_rows):        
        country_group.append(prev_data_hashmap)
        data_hashmap = data_hashmap_array[i]

        if (data_hashmap.get('From country') != prev_data_hashmap.get('From country')):
            # moving on to a new country group
            country_groups.append(country_group)
            country_group = [ ]
            
        prev_data_hashmap = data_hashmap
        i = i + 1
        
    country_group.append(prev_data_hashmap)        
    country_groups.append(country_group)

    return country_groups




def XXXXcreate_from_country_voting_pairs_parent(data_hashmap_array):

    # Example values for header-names
    #   (semi-) final: f, sf
    #   Jury or Televoting: J, T
    #   Year: 1975
    #   To country: Belgium
    #   From country: Belgium
    #   Edition: 1975f, 1975sf

    
    # Sort so array entries are grouped by the country receiving the votes in a given year
    data_hashmap_array.sort(key=xlsxutil.sortkey_year_from_country_voting_type)


    # Debug output
    #
    # for data_hashmap in data_hashmap_array:
    #    print(data_hashmap.get('From country'), data_hashmap.get('Year'), data_hashmap.get('Points'), "(Points to " + data_hashmap.get('To country') + ")")


    # Build array of country groups
    #   A country grouping includes all the votes that country receive that year
    country_groups = []

    prev_data_hashmap = data_hashmap_array[0]
    country_group = [ ]

    i = 1
    num_rows = len(data_hashmap_array)

    while (i < num_rows):        
        country_group.append(prev_data_hashmap)
        data_hashmap = data_hashmap_array[i]

        if (data_hashmap.get('From country') != prev_data_hashmap.get('From country')):
            # moving on to a new country group
            country_groups.append(country_group)
            country_group = [ ]
            
        prev_data_hashmap = data_hashmap
        i = i + 1
        
    country_group.append(prev_data_hashmap)        
    country_groups.append(country_group)

    return country_groups


def debug_print_country_groups(country_groups):
    
    for from_country_voting_array in country_groups:
        # from_country_voting_array => Votes case by given country (in given year) to other countries

        from_country_name = from_country_voting_array[0].get('From country')
        print(from_country_name+":")

        for to_country_entry in from_country_voting_array:
            print("  "+ str(to_country_entry.get('Year')) + str(to_country_entry.get('Points')).rjust(3)
                  +" ("+to_country_entry.get('Jury or Televoting')+") "
                  +" Points to " + to_country_entry.get('To country'))

    
def gs_fileset_voting_by_esc_country_in_year(from_country_year_votes, nul_output_dir_name):

    # Looking to build data-structure (for output as JSON) in the form
    # { "FileSet":
    #   [
    #     { "FileName": "VotesFrom-France1991-J\.nul" }, // The country that cast the votes (J=Jury, T=Tele)
    #     { "Description":
    #       {
    #         "Metadata":
    #           [
    #             { "name": "Germany", "content": 12 }, 
    #             { "name": "Denmark", "content": 6 },  
    #             ...
    #           ]
    #       }
    #     }
    #   ]
    # }


    # Debug output
    
#    for to_country_entry in from_country_year_votes:
#       print("  ", to_country_entry.get('Year'), to_country_entry.get('Points'),
#                 " ("+to_country_entry.get('Jury or Televoting')+") ",
#                 " Points to " + to_country_entry.get('To country'))
            
    metadata_array = []
    metadata_country_vals = []
    metadata_points_vals  = []

    from_country_total = 0

    # Use the first record to be a representative for 'top level' (tl)
    # metadata about the voting 'From country'
    tl_rec = from_country_year_votes[0]
    tl_from_country = tl_rec.get('From country')
    tl_year         = tl_rec.get('Year')
    tl_vote_type    = tl_rec.get('Jury or Televoting')
    tl_from_country_id = "FromCountry-" + re.sub(r'\s+', '', tl_from_country) + str(tl_year) + "-" + tl_vote_type

    vote_mode = xlsxutil.voting_mode(tl_year)
    nuanced_vote_type = xlsxutil.nuanced_voting_type(tl_from_country,tl_year)
    
    metadata_array.append({"name": "Identifier",      "content" : tl_from_country_id})
    metadata_array.append({"name": "FromCountry",     "content" : tl_from_country})
    metadata_array.append({"name": "FromCountryYear", "content" : tl_year})
    metadata_array.append({"name": "FromCountryType", "content" : tl_vote_type})

    metadata_array.append({"name": "FromCountryVoteMode",    "content" : vote_mode})
    metadata_array.append({"name": "FromCountryNuancedType", "content" : nuanced_vote_type})

    if tl_rec.get("FromCountry-Child-Jury") is not None:
        metadata_array.append({"name": "FromCountryChildJury", "content" : tl_rec["FromCountry-Child-Jury"]})
    if tl_rec.get("FromCountry-Child-Tele") is not None:
        metadata_array.append({"name": "FromCountryChildTele", "content" : tl_rec["FromCountry-Child-Tele"]})

    
    for from_country_year_vote in from_country_year_votes:
        to_country   = from_country_year_vote.get('To country')
        year         = from_country_year_vote.get('Year')
        from_country = from_country_year_vote.get('From country')
        vote_type    = from_country_year_vote.get('Jury or Televoting')
        points       = from_country_year_vote.get('Points')

        to_country_year_id = re.sub(r'\s+', '', to_country) + str(year)
            
        voting_rec = { "name": to_country_year_id, "content": points }

        metadata_array.append(voting_rec)
        metadata_country_vals.append(to_country)
        metadata_points_vals.append(points)
        
        from_country_total = from_country_total + points
            
        
    if (len(metadata_country_vals)>0):
        metadata_array.append({ "name": "VotesCountryJSON-"+tl_vote_type,  "content": json.dumps(metadata_country_vals) })
        metadata_array.append({ "name": "VotesPointsJSON-"+tl_vote_type,  "content": json.dumps(metadata_points_vals) })
        # metadata_array.append({ "name": "VotesTotal-"+tl_vote_type, "content": from_country_total})


    # id encodes from-country, year, and vote_type
    filename_id = tl_from_country_id + "\\.nul"

    nul_filename = os.path.join(nul_output_dir_name,tl_from_country_id+".nul");

    print("Creating: " + nul_filename)
    with open(nul_filename, 'w') as outfile:
        outfile.write("")
    
    fileset = {
        "FileSet" : [            
            { "FileName": filename_id, },
            { "Description" : { "Metadata" : metadata_array } }
        ]
    }
              
    return fileset


def gs_directory_metadata(from_country_year_voting_groups):

    # Express the grouped from-country voting data
    # in the Greenstone JSON metadata format:
    
    # { "DirectoryMetadata":
    #   [
    #     { "FileSet":
    #       [
    #         { "FileName": "FromCountry-France1991-J\.nul" },
    #         { "Description":
    #           {
    #             "Metadata":
    #              [
    #                { "name": "Germany", "content": "12" }, 
    #                  ...
    #              ]
    #           }
    #         }
    #       ]
    #     }
    #     ...
    #    ]
    #  }

    nul_output_dir_name = os.path.dirname(json_output_filename)
    
    directory_metadata = []
    for from_country_year_votes in from_country_year_voting_groups:

        fileset = gs_fileset_voting_by_esc_country_in_year(from_country_year_votes, nul_output_dir_name)
        directory_metadata.append(fileset)

        filename_id = fileset.get('FileSet')[0].get('FileName')
        num_countries_voting_data = len(fileset.get('FileSet')[1].get('Description').get('Metadata'))
       
        print("  " + filename_id.ljust(28) + ": " + str(num_countries_voting_data) + " votes")
        

    greenstone_metadata_json = { "DirectoryMetadata": directory_metadata }

    return greenstone_metadata_json


        
if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('--sheetname', help="The name of the sheet within the Excel file to extractc data from")
    parser.add_argument('--votingtype', choices=["J","T", "JT"], help="Filter to only J=Jury, T=Tele cast votes, JT=Combined jury and tele votes")
    parser.add_argument('input-file.xlsx')
    parser.add_argument('output-file.json', nargs='?')
    
    args = parser.parse_args()

    sheetname = getattr(args,'sheetname');
    voting_type = getattr(args,'votingtype');
    
    excel_input_filename = getattr(args,'input-file.xlsx');
    json_output_filename = getattr(args,'output-file.json');

    if (json_output_filename == None):
        json_output_filename = os.path.splitext(excel_input_filename)[0]+'.json'
    
        
    worksheet = xlsxutil.load_xslx_sheet(excel_input_filename,sheetname)

    data_hashmap_array = xlsxutil.convert_worksheet_to_hashmaps(worksheet)
    
    print("Number of data rows in Excel file:   " + str(len(data_hashmap_array)))

    # Filter down to just the voting results concerning finals
    data_hashmap_array_filtered = list(filter(xlsxutil.filter_finalist_votes, data_hashmap_array))

    print("Number of finalist voting data rows: " + str(len(data_hashmap_array_filtered)))

    if voting_type != None:
        # Further filter down by the type of voting results cast
        if (voting_type == "JT"):
            data_hashmap_array_filtered = list_jury_tele_comb_votes(data_hashmap_array_filtered)
            print("Number Jury-Tele *combined* cast data rows: " + str(len(data_hashmap_array_filtered)))
        elif (voting_type == "J"):
            data_hashmap_array_filtered = list(filter(xlsxutil.filter_jury_votes, data_hashmap_array_filtered))
            print("Number Jury cast data rows:          " + str(len(data_hashmap_array_filtered)))
        else:
            # Must be "T"
            data_hashmap_array_filtered = list(filter(xlsxutil.filter_tele_votes, data_hashmap_array_filtered))
            print("Number Televoting cast data rows:    " + str(len(data_hashmap_array_filtered)))

    print()

    from_country_year_voting_groups = {}
    
#    if ((voting_type != None) and (voting_type == "JT")):
#        from_country_year_voting_groups = create_from_country_combined_voting_groups(data_hashmap_array_filtered)
#    else:
#        from_country_year_voting_groups = create_from_country_voting_groups(data_hashmap_array_filtered)

    from_country_year_voting_groups = create_from_country_voting_groups(data_hashmap_array_filtered)
        
    debug_print_country_groups(from_country_year_voting_groups)

    print()
    print("Generating Greenstone JSON from-country voting metadata for:")
    greenstone_metadata_json = gs_directory_metadata(from_country_year_voting_groups)
    
    print("Saving output as: " + json_output_filename)    
    xlsxutil.save_greenstone_json_metadata(greenstone_metadata_json,json_output_filename)

    print()
        
